/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *									     *
 * Program:	subdiv2.5D.c						     *
 *									     *
 * Description:	Demonstrator(TM) of library "libsubdiv2.5D"		     *
 *									     *
 * Date:	20000808						     *
 *									     *
 * Author:	Francisco MORN BURGOS   (http://www.gti.ssr.upm.es./~fmb/)  *
 *		Grupo Tratamiento Imgenes, Univ. Politcnica Madrid, SPAIN  *
 *									     *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */



#include <stdio.h>
#include <stdlib.h>

#include "subdiv2.5D.h"


void parseCmdLine(int argc, char *argv[], sdMeshPtr m, char **mesh_file)
{
  char c;
  int i;
  static char *help[] = {
    "",
    "subdiv2.5D: a polygonal mesh codec based on subdivision surfaces.",
    "",
    "Author: Francisco MORN BURGOS (http://www.gti.ssr.upm.es./~fmb/)",
    "        Grupo Tratamiento Imgenes, Univ. Politcnica Madrid, SPAIN.",
    "",
#ifdef DIME
    "Usage: \"subdiv2.5D [-h] [-v] [-m] [-{t e|p[b]|P[b] {-d|[-b] [-c n]}}]",
#else
    "Usage: \"subdiv2.5D [-h] [-v] [-m] [-{t e|p[b]|P[b] [-b] [-c n]}]",
#endif
    "       mesh_file\", where:",
    "  * \"subdiv2.5D\" must read \"subdiv\" on Windows95/98/NT platforms;",
    "  * \"-h\" prints this help message;",
    "  * \"-v\" turns verbose mode on (\"-v\" turns annoying mode on);",
    "  * \"-m\" uses midpoint subdivision (instead of butterfly scheme);",
    "  * \"-t e\" sets 0.0 < \"e\" < 1.0 as the tolerance for adaptive",
    "    subdivision (based on surface curvature);",
    "  * \"-p\" turns progressive decoding mode on (details read from",
    "    text file \"mesh_file.dt\");",
    "  * \"-pb\" turns progressive decoding mode on (details read from",
    "    binary file \"mesh_file.dtb\");",
    "  * \"-P\" turns progressive encoding mode on (target mesh read from",
    "    \"mesh_file.tgt\"; details written to text file \"mesh_file.dt\");",
    "  * \"-Pb\" turns progressive encoding mode on (target mesh read from",
    "    \"mesh_file.tgt\"; details written to binary file \"mesh_file.dtb\");",
#ifdef DIME
    "  * \"-d\" uses mar's DiMe plugin to project onto the target mesh",
    "    the new midpoints that appear when the current mesh is subdivided;",
#endif
    "  * \"-b\" uses the mesh barycentre to project onto the target mesh",
    "    the new midpoints that appear when the current mesh is subdivided;",
    "  * \"-c\" cuts the target mesh into pieces according to the initial one",
    "    (at most \"n\" nested calls to \"sdlFPBPIP\" are allowed);",
    "  * \"mesh_file\" is the file containing the initial mesh.",
    "",
    0
  };

  while (--argc > 0 && (*++argv)[0] == '-')
    switch (c = *++argv[0]) {
      case 'h': for (i=0; help[i]; i++)
		  printf("%s\n", help[i]);
		exit(0);
                break;
      case 'v': m->verbose = 1;
		break;
      case 'V': m->verbose = 2;
		break;
      case 'm':	m->subdivScheme = MIDPOINT;
		break;
      case 'p':	m->fDetailsIn = TRUE;
      case 'E': m->maxDetailN = 0.5F;  m->maxDetailT = 0.125F;
      case 'P': sdSetProgressiveMode(m, TRUE);
		if (*++argv[0] == 'b') m->fDetailsBinary = TRUE;
		break;
#ifdef DIME
      case 'd': m->useDiMePlugin = TRUE;
		break;
#endif
      case 'b': m->useBarycentre = TRUE;
		break;
      case 'c': m->partition = TRUE;
		m->NmaxNestedCalls2FPBPIP = atoi(*++argv); --argc;
		break;
      case 't': sdSetEpsilon(m, (float) atof(*++argv)); --argc;
		break;
      default:	fprintf(stderr, "Unknown option: \"-%c\".\n", c);
		for (i=0; help[i]; i++)
		  fprintf(stderr, "%s\n", help[i]);
		exit(1);
    }
  if (argc != 1) {
    for (i=0; help[i]; i++)
      fprintf(stderr, "%s\n", help[i]);
    exit(1);
  }
  *mesh_file = argv[0];
} /* parseCmdLine */


int main(int argc, char *argv[])
{
  char *mesh_file, blablabla[88];
  sdMeshPtr m;

  parseCmdLine(argc, argv, m=sdNewMesh(), &mesh_file);
  sdReadInitMesh(m, mesh_file, GUESS);
  sprintf(blablabla, "2D triangular mesh subdivider: \"%s\"", mesh_file);
  sdDisplayMeshForever(m, 512, 512, blablabla);
  return 0;
} /* main */

